// AUTO-GENERATED - DO NOT EDIT. See src/common/tools/gen_listings_and_webworkers.ts.

import { g } from '../../../web_platform/copyToTexture/ImageData.spec.js';
import { wrapTestGroupForWorker } from '../../../../common/runtime/helper/wrap_for_worker.js';

wrapTestGroupForWorker(g);
