// GENERATED CONTENT - DO NOT EDIT
// Content was automatically extracted by Reffy into webref
// (https://github.com/w3c/webref)
// Source: Deprecation Reporting (https://wicg.github.io/deprecation-reporting/)

dictionary DeprecationReportBody : ReportBody {
  DOMString id;
  object? anticipatedRemoval;
  DOMString message;
  DOMString? sourceFile;
  unsigned long? lineNumber;
  unsigned long? columnNumber;
};
