// GENERATED CONTENT - DO NOT EDIT
// Content was automatically extracted by Reffy into webref
// (https://github.com/w3c/webref)
// Source: Intervention Reporting (https://wicg.github.io/intervention-reporting/)

dictionary InterventionReportBody : ReportBody {
  DOMString id;
  DOMString message;
  DOMString? sourceFile;
  unsigned long? lineNumber;
  unsigned long? columnNumber;
};
